SystemOrganization addCategory: #'Phantom-Core'!

ObjectAsMethodWrapper subclass: #PhMethodWrapper
	instanceVariableNames: 'needCleanUp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

PhMethodWrapper subclass: #PhMembraneMethodWrapper
	instanceVariableNames: 'membranes advMembraneCache'
	classVariableNames: 'ProcessMembraneStack'
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhMembraneMethodWrapper classSide methodsFor: 'as yet unclassified' stamp: 'dgg 4/28/2011 17:25'!
ProcessMembraneStack
	^ ProcessMembraneStack ifNil: [ProcessMembraneStack := WeakIdentityKeyDictionary new]! !

!PhMembraneMethodWrapper classSide methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:27'!
currentProcessMembraneStack
	^ self ProcessMembraneStack at: (Processor activeProcess) ifAbsentPut: [OrderedCollection new]! !

!PhMembraneMethodWrapper classSide methodsFor: 'as yet unclassified' stamp: 'dgg 4/28/2011 20:11'!
on: aClass selector: aSelector with: aMembraneArray
	^ (super on: aClass selector: aSelector)
		membranes: aMembraneArray;
		yourself! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 16:10'!
adviceMembranesFor: advGroupIdentifier
	^ advMembraneCache
		at: advGroupIdentifier
		ifAbsentPut: [ 
			advGroupIdentifier = #base
				ifTrue: [ self buildBaseAdviceMembranes ]
				ifFalse: [ self buildAdviceMembranesFor: advGroupIdentifier ] ]! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 15:27'!
buildAdviceMembranesFor: advGroupIdentifier
	| mbrSet aspects advice |
	mbrSet := Set new.
	aspects := Set new.
	advice := Set new.
	advGroupIdentifier do: [ :each | mbrSet addAll: each advisingMembranes ].
	mbrSet do: [ :each | aspects addAll: each registeredAspects ].
	aspects
		do: [ :asp | 
			asp adviceCollection
				do: [ :adv | 
					(adv pointcut includes: self selector on: self wrappedClass)
						ifTrue: [ advice add: adv ] ] ].
	advice := PhAspectWeaver setPrecedenceOn: advice asArray.
	^ Array with: advice with: mbrSet.! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 15:26'!
buildBaseAdviceMembranes
	| aspectSet adv |
	aspectSet := Set new.
	(self membranes select: [ :m | m isBaseMembrane ]) do: [ :bm | aspectSet addAll: bm registeredAspects ].
	adv := OrderedCollection new.
	aspectSet
		do: [ :asp | 
			asp adviceCollection
				do: [ :each | 
					(each pointcut includes: self selector on: self wrappedClass)
						ifTrue: [ adv add: each ] ] ].
	adv := PhAspectWeaver setPrecedenceOn: (PhAspectWeaver setInheritancePrecedenceOn: adv asArray).
	 ^ Array with: adv with: (self membranes select: [ :m | m isBaseMembrane ]).
	! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 16:58'!
cleanCache
	advMembraneCache := Dictionary new.! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 4/29/2011 15:59'!
currentProcessMembraneStack
	^ PhMembraneMethodWrapper currentProcessMembraneStack! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 16:11'!
initialize
	advMembraneCache := Dictionary new.! !

!PhMembraneMethodWrapper methodsFor: 'accessing' stamp: 'dgg 4/28/2011 20:11'!
membranes
	^ membranes! !

!PhMembraneMethodWrapper methodsFor: 'accessing' stamp: 'dgg 4/28/2011 20:12'!
membranes: aMembraneArray
	membranes := aMembraneArray! !

!PhMembraneMethodWrapper methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 15:34'!
run: aSelector with: arguments in: aReceiver
	| membraneStack advMembraneArray |
	membraneStack := PhMembraneMethodWrapper currentProcessMembraneStack.
	membraneStack isEmpty
		ifTrue: [ advMembraneArray := self adviceMembranesFor: #base ]
		ifFalse: [ advMembraneArray := self adviceMembranesFor: membraneStack last ].
	^ (PhAdviceGroup new)
		adviceCollection: (advMembraneArray at: 1);
		advisingMembranes: (advMembraneArray at: 2);
		wrapper: self;
		run: aSelector with: arguments in: aReceiver! !

!PhMethodWrapper methodsFor: 'private' stamp: 'dgg 4/28/2011 16:05'!
currentSender
	"thisContext contextStack inspect."
	(1 to: (thisContext contextStack) size) reverseDo: [:idx |
		(((thisContext contextStack) at: idx) receiver == self) ifTrue: [
			^(thisContext contextStack at: (idx+1)) receiver]].
	self error: 'sender not found'
	"^(thisContext contextStack at: 14) receiver"! !

!PhMethodWrapper methodsFor: 'install-uninstall' stamp: 'dgg 4/28/2011 16:02'!
initialize
	super initialize.
	needCleanUp := false.! !

!PhMethodWrapper methodsFor: 'install-uninstall' stamp: 'dgg 4/28/2011 16:02'!
install
	(self wrappedClass localSelectors includes: self selector) 
		ifTrue: [	needCleanUp := false.]
		ifFalse: [needCleanUp := true].
	"super install."
	oldMethod := self wrappedClass lookupSelector: self selector.
	[oldMethod isKindOf: self class] whileTrue: [oldMethod := oldMethod oldMethod].
	self wrappedClass methodDict at: self selector put: self! !

!PhMethodWrapper methodsFor: 'accessing' stamp: 'dgg 4/28/2011 16:05'!
needCleanUp
	^ needCleanUp! !

!PhMethodWrapper methodsFor: 'accessing' stamp: 'dgg 4/28/2011 16:05'!
needCleanUp: anObject
	needCleanUp := anObject! !

!PhMethodWrapper methodsFor: 'install-uninstall' stamp: 'dgg 4/28/2011 16:02'!
uninstall
	self wrappedClass ifNil: [^self].
	super uninstall.
	needCleanUp ifTrue: [
		SystemChangeNotifier uniqueInstance doSilently: [self wrappedClass removeSelector: self selector]]! !

PPCompositeParser subclass: #PhPrecedenceRulesParser
	instanceVariableNames: 'letter word any identifier anysubclass identifierStar starIdentifierStar starAny'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhPrecedenceRulesParser commentStamp: 'DanielGaldames 4/14/2011 01:54' prior: 0!
This class is responsable to parse the precedence rules defined on a pointcut or aspect.!

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/24/2011 23:13'!
any
	^ $* asParser! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/24/2011 23:13'!
anysubclass
	^$+ asParser! !

!PhPrecedenceRulesParser methodsFor: 'node set generation' stamp: 'DanielGaldames 3/25/2011 12:17'!
buildNodeSetFrom: aParserNode
	| aSet |
	aSet := Set new.
	((aParserNode at: 1) matches: (PhAspect name asString)) ifTrue: [aSet add: PhAspect].
	PhAspect allSubclasses do: [:asp |
		((aParserNode at: 1) matches: (asp name asString)) ifTrue: [aSet add: asp]].
	(aParserNode at: 2) notNil ifTrue: [
		| subs |
		subs := Set new.
		aSet do: [:asp | asp allSubclasses do: [:sub | subs add: sub]].
		subs do: [:asp | aSet add: asp.]].
	^aSet collect: [:asp | AspectNode new aspect: asp].! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/24/2011 23:55'!
identifier
	^ (letter, word star) flatten! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/25/2011 11:56'!
identifierStar
	^ (identifier, any optional, anysubclass optional) ==> [:node |
		| precedenceParser|
		precedenceParser := (node at: 1) asParser.
		(node at: 2) notNil ifTrue: [precedenceParser := precedenceParser, #word asParser star].
		Array with: precedenceParser with: (node at: 3)]! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/24/2011 23:10'!
letter
	^ #letter asParser! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/29/2011 22:09'!
starAny
	^ (any) ==> [:node | Array with: (#word asParser star) with: nil]! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/25/2011 12:21'!
starIdentifierStar
	^ (any, word plus flatten, any optional, anysubclass optional) ==> [:node |
		| precedeceParser |
		precedeceParser := (#word asParser starGreedy: ((node at: 2) asParser)), (node at: 2) asParser.
		(node at: 3) notNil ifTrue: [precedeceParser := precedeceParser, #word asParser star].
		Array with: precedeceParser with: (node at: 4)]! !

!PhPrecedenceRulesParser methodsFor: 'parser' stamp: 'DanielGaldames 3/25/2011 11:46'!
start
	^ (identifierStar / starIdentifierStar / starAny) end ==> [:node | self buildNodeSetFrom: node  ]! !

!PhPrecedenceRulesParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/24/2011 23:56'!
word
	^ #word asParser! !

PPCompositeParser subclass: #PhReceiverParser
	instanceVariableNames: 'starToken nameToken plusToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

PhReceiverParser class
	instanceVariableNames: 'classAndSubclassAction subclassesAction metaNameAction nameAction'!

!PhReceiverParser commentStamp: 'DanielGaldames 4/14/2011 01:55' prior: 0!
this class is responsable to parse the receivers declared on a pointcut.!

PhReceiverParser class
	instanceVariableNames: 'classAndSubclassAction subclassesAction metaNameAction nameAction'!

!PhReceiverParser methodsFor: 'class parser generation' stamp: 'DanielGaldames 3/21/2011 14:51'!
generateReceiverClassParser: aParserNode
	| aNameParser classParser |
	aNameParser := ((aParserNode at: 2) at: 1) asParser.
	(aParserNode at: 1) notNil ifTrue: [aNameParser := (#word asParser starGreedy: aNameParser), aNameParser].
	((aParserNode at: 2) at: 2) notNil ifTrue: [aNameParser := aNameParser, #word asParser star].
	classParser := #word asParser plus, (#space asParser plus), 'subclass:' asParser, (#space asParser plus), $# asParser, aNameParser, (#blank asParser | #newline asParser).
	"subclassParser := aNameParser, #space asParser plus, 'subclass:' asParser."
	(aParserNode at: 3) notNil
		ifTrue: [ | matchedClasses subclassParser |
			matchedClasses := (SmalltalkImage current allClasses) select: [:each |
				classParser matches: each definition].
			matchedClasses do: [:each |
				each allSubclasses do: [:s | classParser := classParser / s definition asParser]].
			^classParser]
		ifFalse: [^classParser]
	! !

!PhReceiverParser methodsFor: 'class parser generation' stamp: 'DanielGaldames 3/21/2011 14:47'!
generateReceiverMetaclassParser: aParserNode
	| aNameParser metaclassParser |
	aNameParser := ((aParserNode at: 2) at: 1) asParser.
	(aParserNode at: 1) notNil ifTrue: [aNameParser := (#word asParser starGreedy: aNameParser), aNameParser].
	((aParserNode at: 2) at: 2) notNil ifTrue: [aNameParser := aNameParser, #word asParser star].
	metaclassParser := aNameParser, #space asParser plus, 'class' asParser,(#blank asParser | #newline asParser).
	(aParserNode at: 3) notNil
		ifFalse: [^metaclassParser]
		ifTrue: [
			| matchedMetaclasses |
			matchedMetaclasses := (SmalltalkImage current allClasses) collect: [:each | each class] thenSelect: [:each |
				metaclassParser matches: each definition].
			matchedMetaclasses do: [:each |
				each allSubclasses do: [:s | metaclassParser := metaclassParser / s definition asParser]].
			^metaclassParser.]! !

!PhReceiverParser methodsFor: 'class parser generation' stamp: 'DanielGaldames 3/17/2011 21:02'!
generateReceiverParser: aParseNode
	^(((aParseNode at: 2) at: 3) notNil)
		ifTrue: [self generateReceiverMetaclassParser: aParseNode]
		ifFalse: [self generateReceiverClassParser: aParseNode]! !

!PhReceiverParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/16/2011 00:05'!
nameToken
	^((#letter asParser, #word asParser plus) flatten, starToken optional, ' class' asParser optional).! !

!PhReceiverParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/15/2011 23:14'!
plusToken
	^$+ asParser! !

!PhReceiverParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/15/2011 23:13'!
starToken
	^$* asParser! !

!PhReceiverParser methodsFor: 'parser' stamp: 'DanielGaldames 3/17/2011 20:38'!
start
	^((starToken optional, nameToken, plusToken optional) end) ==> [:node |
		self generateReceiverParser: node.]! !

PPCompositeParser subclass: #PhSelectorParser
	instanceVariableNames: 'identifier keyword unary binary multiword anyKeyUnary anyKeyMulti'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhSelectorParser commentStamp: 'DanielGaldames 4/14/2011 01:56' prior: 0!
this class is responsable to parse the selectors declared on a pointcut.!

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 18:50'!
anyKeyMulti
	^($_ asParser, $: asParser) ==> [:node | self identifier, $: asParser].! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 19:09'!
anyKeyUnary
	^($_ asParser, $: asParser not) ==> [:node | self identifier].! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 17:03'!
binary
	^ ((PPPredicateObjectParser anyOf: '!!%&*+,-/<=>?@\|~') ,
	  (PPPredicateObjectParser anyOf: '!!%&*+,/<=>?@\|~') star) flatten token ==> [:node | node value asParser]! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 14:12'!
identifier
	^ #letter asParser, #word asParser star! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 17:02'!
keyword
	^ (identifier , $: asParser) token ==> [:node | node value asParser].! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 18:24'!
multiword
	^ (keyword | anyKeyMulti) plus! !

!PhSelectorParser methodsFor: 'parser' stamp: 'DanielGaldames 3/18/2011 19:05'!
start
	^ (unary / anyKeyUnary  / binary / multiword) end ==> [:node |
		(node isCollection) 
			ifTrue: [
				(node size > 1) 
					ifTrue:  [| p |
						p := node reduce: [:a :b | a,b].
						p end] 
					ifFalse: [(node at: 1) end]  ]
			ifFalse: [node end] ]! !

!PhSelectorParser methodsFor: 'tokens' stamp: 'DanielGaldames 3/18/2011 17:03'!
unary
	^ (identifier, $: asParser not) token ==> [:node | node value asParser]! !

Object subclass: #PhAdvice
	instanceVariableNames: 'type pointcut advice owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAdvice commentStamp: 'DanielGaldames 4/13/2011 23:36' prior: 0!
PhAdvice represents a pointcut, advice pair.
Use pointcut:  to declare the joinpoint to capture, using an instance of PhPointcut.
Use type: to declare the type of the advice,
	#before
	#after
	#around.
Use advice: to add the method to execute on the captued joinpoints.
advice: accept a block or a selector, with 0 arguments (if no context is captured on the pointcut), or 1 arguments for the context captured on the joinpoint.
you can also use selector:of: to send a message on an object for advice.
Example: 
	PhAdvice new
		pointcut: aPointcut;
		type: #before;
		advice: [:context | Transcript show: context receiver asString].
		
	PhAdvice new
		pointcut: aPointcut;
		type: #around;
		selector: #theAdviceOnFoo: of: foo.  !

!PhAdvice methodsFor: 'accessing' stamp: 'dgg 4/26/2011 18:59'!
advice: aBlockOrSymbolBlockArray
	advice := aBlockOrSymbolBlockArray.! !

!PhAdvice methodsFor: 'private' stamp: 'DanielGaldames 3/25/2011 22:15'!
owner
	^ owner ! !

!PhAdvice methodsFor: 'private' stamp: 'DanielGaldames 3/25/2011 22:14'!
owner: anAspect
	owner := anAspect! !

!PhAdvice methodsFor: 'accessing' stamp: 'DanielGaldames 3/8/2011 16:50'!
pointcut
	^ pointcut! !

!PhAdvice methodsFor: 'accessing' stamp: 'DanielGaldames 3/8/2011 16:50'!
pointcut: aPhPointcut
	pointcut := aPhPointcut! !

!PhAdvice methodsFor: 'private' stamp: 'dgg 4/26/2011 18:59'!
run
	(advice isBlock) ifTrue: [^(advice value) ].
	^((advice at: 1) perform: (advice at: 2))
	    ! !

!PhAdvice methodsFor: 'private' stamp: 'dgg 4/26/2011 18:59'!
runWithin: aContext
	(advice isBlock) ifTrue: [^(advice value: aContext) ].
	^((advice at: 1) perform: (advice at: 2) with: aContext)
	    ! !

!PhAdvice methodsFor: 'accessing' stamp: 'dgg 5/2/2011 15:13'!
send: aSymbol to: anObject
	self advice: (Array with: anObject with: aSymbol) ! !

!PhAdvice methodsFor: 'accessing' stamp: 'DanielGaldames 3/8/2011 16:51'!
type
	^ type ifNil: [type := #before]! !

!PhAdvice methodsFor: 'accessing' stamp: 'DanielGaldames 3/8/2011 16:51'!
type: aSymbol
	type := aSymbol! !

Object subclass: #PhAdviceGroup
	instanceVariableNames: 'beforeAdvice aroundAdvice afterAdvice currentAdviceIndex wrapper advisingMembranes adviceCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAdviceGroup methodsFor: 'as yet unclassified' stamp: 'dgg 4/24/2011 21:37'!
adviceCollection: anAdviceCollection
	adviceCollection := anAdviceCollection.
	beforeAdvice := (anAdviceCollection select: [:adv | adv type = #before]) asArray.
	aroundAdvice := (anAdviceCollection select: [:adv | adv type = #around]) asArray.
	afterAdvice := (anAdviceCollection select: [:adv | adv type = #after]) asArray reversed.! !

!PhAdviceGroup methodsFor: 'as yet unclassified' stamp: 'dgg 4/29/2011 15:44'!
advisingMembranes
	^ advisingMembranes! !

!PhAdviceGroup methodsFor: 'as yet unclassified' stamp: 'dgg 4/29/2011 15:44'!
advisingMembranes: aMembraneSet
	advisingMembranes := aMembraneSet! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 17:43'!
afterAdvice
	^ afterAdvice! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 17:43'!
afterAdvice: anAdviceCollection
	afterAdvice := anAdviceCollection.
	afterAdvice do: [:each | each type: #after]! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 18:36'!
aroundAdvice
	^ aroundAdvice! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 17:43'!
aroundAdvice: anAdviceCollection
	aroundAdvice := anAdviceCollection.
	aroundAdvice do: [:each | each type: #around]! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 17:43'!
beforeAdvice
	^ beforeAdvice! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 17:43'!
beforeAdvice: anAdviceCollection
	beforeAdvice := anAdviceCollection.
	beforeAdvice do: [:each | each type: #before]! !

!PhAdviceGroup methodsFor: 'private' stamp: 'dgg 4/26/2011 15:55'!
buildAdviceGroupFor: aContext
	aContext adviceDictionary at: #before put: beforeAdvice.
	aContext adviceDictionary at: #after put: afterAdvice.
	aContext adviceDictionary at: #around put: aroundAdvice.! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/26/2011 15:47'!
oldMethod
	^ self wrapper oldMethod! !

!PhAdviceGroup methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:41'!
run: aSelector with: arguments in: aReceiver
	| aResult |
	PhBeforeAdviceRunner new
		selector: aSelector;
		receiver: aReceiver;
		arguments: arguments;
		adviceGroup: self;
		advice: self beforeAdvice;
		run.
	aResult := PhAroundAdviceRunner new
		selector: aSelector;
		receiver: aReceiver;
		arguments: arguments;
		adviceGroup: self;
		advice: self aroundAdvice;
		run.
	PhAfterAdviceRunner new
		selector: aSelector;
		receiver: aReceiver;
		arguments: arguments;
		adviceGroup: self;
		advice: self afterAdvice;
		run.
	^aResult! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/24/2011 18:38'!
wrapper
	^wrapper! !

!PhAdviceGroup methodsFor: 'accessing' stamp: 'dgg 4/29/2011 13:10'!
wrapper: aPhMethodMethodWrapper
	wrapper := aPhMethodMethodWrapper! !

Object subclass: #PhAdviceRunner
	instanceVariableNames: 'advice index type selector receiver arguments adviceGroup membraneSetIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:24'!
addMembranes
	| membraneSet |
	membraneSet := Set new.
	self adviceGroup wrapper currentProcessMembraneStack add: membraneSet.
	membraneSetIndex := self adviceGroup wrapper currentProcessMembraneStack size.! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:22'!
addMembranesOf: currentAdvice
	| membraneSet |
	membraneSet := Set new.
	membraneSet
		addAll: (self adviceGroup advisingMembranes select: [ :mbr | mbr registeredAspects includes: currentAdvice owner ]).
	(self adviceGroup wrapper currentProcessMembraneStack) at: membraneSetIndex put: membraneSet! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/25/2011 08:55'!
advice
	^ advice! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/25/2011 08:55'!
advice: anObject
	advice := anObject! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:23'!
adviceGroup
	^ adviceGroup! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:23'!
adviceGroup: anObject
	adviceGroup := anObject! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:09'!
arguments
	^ arguments! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:13'!
arguments: anObject
	arguments := anObject! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 15:56'!
buildAdviceGroupFor: aContext
	self adviceGroup buildAdviceGroupFor: aContext.
	aContext adviceRunner: self.      ! !

!PhAdviceRunner methodsFor: 'private' stamp: 'dgg 5/6/2011 01:00'!
buildContextFor: anAdvice
	| aContext contextExp buildAdviceContext |
	contextExp := (anAdvice pointcut context) collect: [:each | (each = #proceed) ifTrue: [#method] ifFalse: [each]].
	buildAdviceContext := contextExp includes: #advice.
	aContext := PhContext new.
	(buildAdviceContext) ifTrue: [
		contextExp := contextExp select: [:each | (each = #advice) not].
		self buildAdviceGroupFor: aContext..].
	(contextExp) do: [:each |
		 aContext perform: (each asString,':') asSymbol with: (self perform: each asSymbol)].
	(contextExp includes: #method) ifTrue: [
		aContext arguments: self arguments.
		aContext receiver: self receiver.
		aContext selector: self selector].
	(anAdvice pointcut dynamicContext) ifNotNilDo: [:block |
		block value: aContext value: thisContext].
	aContext adviceRunner: self.
	^ aContext! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 16:06'!
changeAdvice: anAdviceCollection continueAt: anIndex
	advice := anAdviceCollection.
	index := anIndex -1! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/25/2011 08:55'!
index
	^ index! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/25/2011 08:55'!
index: anObject
	index := anObject! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/3/2011 21:49'!
initialize
	super initialize.
	index := 1! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:24'!
method
	^ self adviceGroup wrapper oldMethod.! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 01:45'!
proceed
	(index < advice size)
		ifFalse: [^ self method valueWithReceiver: receiver arguments: arguments]
		ifTrue: [^ self runNext]! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 01:45'!
proceed: changedArguments
	self arguments: changedArguments.
	(index < advice size)
		ifFalse: [^ self method valueWithReceiver: receiver arguments: arguments]
		ifTrue: [^ self runNext]! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:09'!
receiver
	^ receiver! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:13'!
receiver: anObject
	receiver := anObject! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/29/2011 16:04'!
removeMembranes
	self adviceGroup wrapper currentProcessMembraneStack removeLast.! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:17'!
run
	| result |
	self addMembranes.
	result := self runAdvice.
	self removeMembranes.
	^ result! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:20'!
runAdvice
	[ index <= advice size ]
		whileTrue: [ 
			| currentAdvice |
			currentAdvice := advice at: index.
			self addMembranesOf: currentAdvice.
			currentAdvice pointcut context notEmpty | currentAdvice pointcut dynamicContext notNil
				ifTrue: [ currentAdvice runWithin: (self buildContextFor: currentAdvice) ]
				ifFalse: [ currentAdvice run ].
			index := index + 1 ]! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 02:47'!
runNext
	| result currentAdvice currentMembranes |
	currentMembranes := self adviceGroup wrapper currentProcessMembraneStack at: membraneSetIndex.
	index := index + 1.
	currentAdvice := advice at: index.
	self addMembranesOf: currentAdvice.
	currentAdvice pointcut context notEmpty | currentAdvice pointcut dynamicContext notNil
		ifTrue: [ result := currentAdvice runWithin: (self buildContextFor: currentAdvice) ]
		ifFalse: [ result := currentAdvice run ].
	self adviceGroup wrapper currentProcessMembraneStack at: membraneSetIndex put: currentMembranes.
	^ result! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:09'!
selector
	^ selector! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/26/2011 13:13'!
selector: anObject
	selector := anObject! !

!PhAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:24'!
sender
	^ self adviceGroup wrapper currentSender.! !

!PhAdviceRunner methodsFor: 'accessing' stamp: 'dgg 4/25/2011 09:31'!
type
	^ self subclassResponsibility! !

PhAdviceRunner subclass: #PhAfterAdviceRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAfterAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:21'!
changeAdvice: anAdviceCollection continueAt: anIndex
	super changeAdvice: anAdviceCollection continueAt: anIndex.
	adviceGroup afterAdvice: anAdviceCollection.! !

!PhAfterAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/25/2011 09:33'!
type
	^#after! !

PhAdviceRunner subclass: #PhAroundAdviceRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAroundAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:21'!
changeAdvice: anAdviceCollection continueAt:  anIndex
	super changeAdvice: anAdviceCollection continueAt: anIndex.
	adviceGroup aroundAdvice: anAdviceCollection.! !

!PhAroundAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 5/4/2011 00:25'!
runAdvice
	| aResult |
	advice notEmpty
		ifTrue: [ 
			[ index <= advice size ]
				whileTrue: [ 
					| currentAdvice |
					currentAdvice := advice at: index.
					self addMembranesOf: currentAdvice.  
					currentAdvice pointcut context notEmpty | currentAdvice pointcut dynamicContext notNil
						ifTrue: [ aResult := currentAdvice runWithin: (self buildContextFor: currentAdvice) ]
						ifFalse: [ aResult := currentAdvice run ].
					index := index + 1] ]
		ifFalse: [ aResult := adviceGroup oldMethod valueWithReceiver: self receiver arguments: self arguments ].
	^ aResult! !

!PhAroundAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/25/2011 09:33'!
type
	^#around! !

PhAdviceRunner subclass: #PhBeforeAdviceRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhBeforeAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/26/2011 13:21'!
changeAdvice: anAdviceCollection continueAt:  anIndex
	super changeAdvice: anAdviceCollection continueAt: anIndex.
	adviceGroup afterAdvice: anAdviceCollection.! !

!PhBeforeAdviceRunner methodsFor: 'as yet unclassified' stamp: 'dgg 4/25/2011 09:33'!
type
	^#before! !

Object subclass: #PhAspect
	instanceVariableNames: 'adviceCollection precedence classModifiers installed onMembranes membrane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAspect commentStamp: 'DanielGaldames 4/13/2011 23:50' prior: 0!
PhAspect represent an Aspect, it has a collection of PhAdvices, an PhClassModifiers.
Use add: to add advices.
Use addClassModifier: to add a classModifier.
Use precedence: to specify the desired precedence on the installed aspects.
Use install to install it, and uninstall to uninstall the aspect.
Example:
	PhAspect new
		add: (PhAdvice new ...);
		addClassMofifier: (PhClassModifier new on: ...);
		precedence: 'PhAspectBigger+ PhAspectFoo'.!

!PhAspect methodsFor: 'accessing' stamp: 'dgg 5/2/2011 15:02'!
add: anAdvice
	anAdvice owner: self.
	adviceCollection add: anAdvice.
	PhAspectWeaver cleanCache.! !

!PhAspect methodsFor: 'accessing' stamp: 'DanielGaldames 4/13/2011 22:39'!
addClassModifier: aClassModifier
	classModifiers add: aClassModifier.! !

!PhAspect methodsFor: 'accessing' stamp: 'DanielGaldames 3/20/2011 16:50'!
adviceCollection
	^ adviceCollection! !

!PhAspect methodsFor: 'initialization' stamp: 'dgg 5/1/2011 21:17'!
initialize
	adviceCollection := OrderedCollection new.
	classModifiers := Set new.
	installed := false.
	onMembranes := Set new.! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 14:30'!
install
	| pointcutSet pointcut |
	pointcutSet := Set new.
	self adviceCollection do: [:each |
		pointcutSet add: each pointcut].
	pointcutSet isEmpty ifFalse: [
		(pointcutSet size < 2) 
			ifTrue: [pointcut := pointcutSet asArray at: 1]
			ifFalse: [
				pointcut := pointcutSet asArray reduce: [:a :b |
					a por: b]]].
	membrane := PhMembrane new pointcut: pointcut.
	membrane install.
	self registerOn: membrane.    ! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 13:48'!
installClassModification
	installed ifFalse: [
		classModifiers do: [:each | each install].
		installed := true]! !

!PhAspect methodsFor: 'private' stamp: 'dgg 5/1/2011 21:25'!
onMembranes
	^ onMembranes! !

!PhAspect methodsFor: 'private' stamp: 'dgg 5/1/2011 21:25'!
onMembranes: anObject
	onMembranes := anObject! !

!PhAspect methodsFor: 'accessing' stamp: 'DanielGaldames 3/25/2011 20:13'!
precedence
	^ precedence ifNil: [precedence := #()]  ! !

!PhAspect methodsFor: 'accessing' stamp: 'DanielGaldames 3/25/2011 20:13'!
precedence: aPrecedenceArray
	precedence := aPrecedenceArray! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 13:50'!
registerOn: aMembrane
	aMembrane registerAspect: self.
	onMembranes add: aMembrane.
	self installClassModification.! !

!PhAspect methodsFor: 'accessing' stamp: 'dgg 5/2/2011 15:02'!
remove: anAdvice
	adviceCollection remove: anAdvice.
	PhAspectWeaver cleanCache! !

!PhAspect methodsFor: 'accessing' stamp: 'DanielGaldames 4/13/2011 22:39'!
removeClassModifier: aClassModifier
	classModifiers remove: aClassModifier! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 14:44'!
uninstall
	self unregisterFrom: membrane.
	membrane uninstall.! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 13:49'!
uninstallClassModification
	"PhAspectWeaver unInstall: self"
	installed ifTrue: [
		classModifiers do: [:each | each uninstall].
		installed := false]! !

!PhAspect methodsFor: 'install-uninstall' stamp: 'dgg 5/2/2011 14:56'!
unregisterFrom: aMembrane
	aMembrane unregisterAspect: self.
	onMembranes remove: aMembrane ifAbsent: nil.
	onMembranes isEmpty ifTrue: [self uninstallClassModification]! !

Object subclass: #PhAspectWeaver
	instanceVariableNames: ''
	classVariableNames: 'InstalledAspects WrapperList installedMembranes'
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhAspectWeaver commentStamp: 'DanielGaldames 4/14/2011 01:11' prior: 0!
This class is responsable to weave the aspects into the system.!

!PhAspectWeaver classSide methodsFor: 'initialize-release' stamp: 'DanielGaldames 4/11/2011 01:29'!
activateSystemNotification
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #class
		change: #Added
		using: #systemEvent:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #class
		change: #Modified
		using: #systemEvent:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #class
		change: #Removed
		using: #systemEvent:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #class
		change: #Renamed
		using: #classRename:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #method
		change: #Added
		using: #systemEvent:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #method
		change: #Modified
		using: #methodModification:.
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #method
		change: #Removed
		using: #methodRemoved:! !

!PhAspectWeaver classSide methodsFor: 'accessors' stamp: 'dgg 4/28/2011 14:05'!
addMembrane: aMembrane
	self installedMembranes add: aMembrane.. ! !

!PhAspectWeaver classSide methodsFor: 'notification' stamp: 'dgg 5/1/2011 15:23'!
classRename: anEvent
	anEvent item methodDict do: [:each |
		(each isKindOf: PhMethodWrapper)
			ifTrue: [
				each reference setClass: (anEvent item) methodSymbol: (each reference methodSymbol)]].
	self updateMembranes.! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'dgg 5/1/2011 15:08'!
cleanCache
	WrapperList do: [:each | each cleanCache]! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'DanielGaldames 4/10/2011 21:23'!
cleanWrappers
	WrapperList notEmpty 
		ifTrue: [
			WrapperList do: [:each | each uninstall ].
			WrapperList removeAll].! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'DanielGaldames 3/29/2011 12:52'!
combineOrderOf: aSortedAdvicesCollection and: anotherSortedAdviceCollection
	| combinedCollection rest |
	combinedCollection := OrderedCollection new.
	rest := OrderedCollection new.
	combinedCollection addAll: aSortedAdvicesCollection.
	anotherSortedAdviceCollection reverseDo: [:each |
		(combinedCollection includes: each) 
			ifTrue: [	rest do: [:adv | combinedCollection add: adv after: each]. rest removeAll]
			ifFalse: [rest addFirst: each]].
	combinedCollection addAllFirst: rest.
	^combinedCollection  ! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'dgg 5/1/2011 15:15'!
compileAndInstallMembranes
	| pointcutDict aspectRulesDict |
	pointcutDict := Dictionary new.
	"self installedAspects do: [:aspect | 
		aspect adviceCollection do: [:advice |
			advice pointcut getPointcuts do: [:pointcut |
				pointcutDict at: pointcut
					ifPresent: [:ascol |
						(ascol includes: advice) ifFalse: [ascol add: advice]]
					ifAbsent: [| ascol |
						ascol := OrderedCollection new.
						ascol add: advice.
						pointcutDict at: pointcut put: ascol] ]]].
	WrapperList := self getWrapperListWith: pointcutDict.

	WrapperList do: [:each | each install]"
	self installedMembranes do: [:membrane |
		membrane pointcut getPointcuts do: [:pointcut |
			pointcutDict at: pointcut
				ifPresent: [:membraneCollection |
					(membraneCollection includes: membrane) ifFalse: [membraneCollection add: membrane]]
				ifAbsent: [pointcutDict at: pointcut put: (OrderedCollection with: membrane)]]].
	WrapperList := self getMembraneWrapperListWith: pointcutDict.
	WrapperList do: [:each | each install]! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'dgg 5/1/2011 15:15'!
getMembraneWrapperListWith: aPointcutDictionary
	| dict |
	dict := Dictionary new.
	aPointcutDictionary keysAndValuesDo: [:key :value |
		dict at: key put: (PhMembraneMethodWrapper on: (key at: 1) selector: (key at: 2) with: value asArray)].
	^dict  ! !

!PhAspectWeaver classSide methodsFor: 'initialize-release' stamp: 'DanielGaldames 4/11/2011 01:29'!
initialize
	super initialize.
	self activateSystemNotification.
	WrapperList := Dictionary new.! !

!PhAspectWeaver classSide methodsFor: 'install/uninstall' stamp: 'dgg 5/1/2011 15:20'!
installMembrane: aMembrane
	self cleanWrappers.
	self addMembrane: aMembrane.
	self compileAndInstallMembranes! !

!PhAspectWeaver classSide methodsFor: 'accessors' stamp: 'dgg 5/4/2011 16:56'!
installedAspects
	| aspectSet |
	aspectSet := Set new.
	installedMembranes do: [:e | 
		aspectSet addAll: e registeredAspects].
	^ aspectSet.! !

!PhAspectWeaver classSide methodsFor: 'accessors' stamp: 'dgg 5/1/2011 15:18'!
installedMembranes
	^ installedMembranes ifNil: [installedMembranes := Set new.]! !

!PhAspectWeaver classSide methodsFor: 'notification' stamp: 'dgg 5/1/2011 15:23'!
methodModification: anEvent
	(anEvent oldItem isKindOf: PhMethodWrapper)
		ifTrue: [ 
			anEvent oldItem oldMethod: anEvent item.
			anEvent oldItem needCleanUp: false.
			anEvent oldItem install ]! !

!PhAspectWeaver classSide methodsFor: 'notification' stamp: 'dgg 5/1/2011 15:23'!
methodRemoved: anEvent
	(anEvent item isKindOf: PhMethodWrapper)
		ifTrue: [WrapperList removeKey: (Array with: (anEvent itemClass) with: (anEvent item selector) )].
	self updateMembranes! !

!PhAspectWeaver classSide methodsFor: 'accessors' stamp: 'dgg 4/28/2011 13:40'!
removeMembrane: aMembrane
	self installedMembranes remove: aMembrane ifAbsent: nil.! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'DanielGaldames 4/19/2011 15:53'!
setInheritancePrecedenceOn: anAdviceCollection
	| aspectList adviceList workingSet tmp |
	aspectList := Set new.
	aspectList add: PhAspect.
	aspectList addAll: PhAspect allSubclasses.
	adviceList := OrderedCollection new.
	adviceList addAll: (anAdviceCollection select: [:adv | adv owner class theNonMetaClass = PhAspect]).
	workingSet := OrderedCollection with: PhAspect.
	tmp := OrderedCollection new.
	[aspectList notEmpty] whileTrue: [
		tmp removeAll.
		workingSet do: [:father |
			tmp addAll: (aspectList select: [:asp | asp superclass = father]).
			aspectList remove: father].
		tmp do: [:asp |
				adviceList addAll: (anAdviceCollection select: [:adv | adv owner class theNonMetaClass = asp])].
		workingSet removeAll.
		workingSet addAll: tmp].
	^ adviceList.! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'DanielGaldames 3/29/2011 12:09'!
setPrecedenceOn: anAdviceSet
	"ordenar basado en precedencia"
	| pointcutRulesSet pointcutOverrideRuleSet aspectRulesSet sortedAspects sortedAdvices |
	aspectRulesSet := Set new.
	pointcutRulesSet := Set new.
	pointcutOverrideRuleSet := Set new.
	anAdviceSet do: [:adv |
		(adv owner precedence notEmpty) ifTrue: [aspectRulesSet add: adv owner precedence].
		(adv pointcut precedence notEmpty) ifTrue: [
			(adv pointcut override) 
				ifFalse: [pointcutRulesSet add: adv pointcut precedence]
				ifTrue: [pointcutOverrideRuleSet add: adv pointcut precedence]]].
	((aspectRulesSet isEmpty) & (pointcutRulesSet isEmpty) & (pointcutOverrideRuleSet isEmpty)) ifTrue: [^anAdviceSet].
	(pointcutOverrideRuleSet isEmpty) 
		ifFalse: [
			sortedAspects := PhPrecedenceRulesProcessor new getSortedAspectsWith: pointcutOverrideRuleSet.  
			sortedAdvices := self sort: anAdviceSet from: sortedAspects.  
			(anAdviceSet difference: sortedAdvices) do: [:e | sortedAdvices add: e].
			^sortedAdvices. ]
		ifTrue: [
			sortedAspects := PhPrecedenceRulesProcessor new getSortedAspectsWith: pointcutRulesSet.
			sortedAdvices := self sort: anAdviceSet from: sortedAspects.
			sortedAspects := PhPrecedenceRulesProcessor new getSortedAspectsWith: aspectRulesSet.
			sortedAdvices := self combineOrderOf: sortedAdvices and: (self sort: anAdviceSet from: sortedAspects).
			^sortedAdvices]! !

!PhAspectWeaver classSide methodsFor: 'private' stamp: 'DanielGaldames 3/29/2011 12:44'!
sort: anAdvicesCollection from: aSortedAspectCollection
	| sortedAdvices |
	sortedAdvices := OrderedCollection new.
	aSortedAspectCollection do: [:each |
		| col |
		col := anAdvicesCollection select: [:adv | adv owner class = each].
		col do: [:adv | sortedAdvices add: adv]].
	^sortedAdvices.! !

!PhAspectWeaver classSide methodsFor: 'notification' stamp: 'dgg 5/1/2011 15:24'!
systemEvent: anEvent
	self updateMembranes.! !

!PhAspectWeaver classSide methodsFor: 'install/uninstall' stamp: 'dgg 5/1/2011 15:20'!
uninstallAll
	self installedMembranes removeAll.
	self cleanWrappers.  ! !

!PhAspectWeaver classSide methodsFor: 'install/uninstall' stamp: 'dgg 5/1/2011 15:20'!
uninstallMembrane: aMembrane
	self cleanWrappers.
	self removeMembrane: aMembrane.
	self compileAndInstallMembranes! !

!PhAspectWeaver classSide methodsFor: 'update' stamp: 'dgg 5/1/2011 15:21'!
updateMembranes
	self cleanWrappers.
	self compileAndInstallMembranes! !

Object subclass: #PhClassModifier
	instanceVariableNames: 'modifiedClass newInstanceMethodDict newClassMethodDict newInstanceVar newClassVar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhClassModifier commentStamp: 'DanielGaldames 4/14/2011 01:22' prior: 0!
This class is use to declare the modification to do on a class, at the moment of install an aspect.
You can add instance variables, class variables, instance methods and class side methods.
This modifications are rolled back on uninstall.!

!PhClassModifier methodsFor: 'method-manipulation' stamp: 'dgg 4/23/2011 02:23'!
addNewClassMethod: aMethodString
	| aSelector |
	aSelector := (RBParser parseMethod: aMethodString) selector.
	(modifiedClass theMetaClass allSelectors includes: aSelector)
		ifFalse: 	[newClassMethodDict add: (aSelector -> aMethodString)]
		ifTrue: [self error: 'method already defined']! !

!PhClassModifier methodsFor: 'variable-manipulation' stamp: 'DanielGaldames 4/13/2011 21:18'!
addNewClassVar: aClassVarName
	| varName |
	varName := aClassVarName capitalized.
	(modifiedClass hasClassVarNamed: varName) 
		ifFalse:	[newClassVar add: varName]
		ifTrue: [self error: 'variable already defined']! !

!PhClassModifier methodsFor: 'method-manipulation' stamp: 'dgg 4/23/2011 02:23'!
addNewInstanceMethod: aMethodString
	| aSelector |
	aSelector := (RBParser parseMethod: aMethodString) selector.
	(modifiedClass allSelectors includes: aSelector)
		ifFalse: 	[newInstanceMethodDict add: (aSelector -> aMethodString)]
		ifTrue: [self error: 'method already defined']! !

!PhClassModifier methodsFor: 'variable-manipulation' stamp: 'DanielGaldames 4/13/2011 20:31'!
addNewInstanceVar: anInstanceVarName
	(modifiedClass hasInstVarNamed: anInstanceVarName) 
		ifFalse:	[newInstanceVar add: anInstanceVarName]
		ifTrue: [self error: 'variable already defined']! !

!PhClassModifier methodsFor: 'private' stamp: 'dgg 4/23/2011 02:24'!
checkExistence
	newInstanceVar do: [:varName |
		(modifiedClass theNonMetaClass hasInstVarNamed: varName) ifTrue: [self error: 'instance variable already defined!!']].
	newClassVar do: [:varName |
		(modifiedClass theNonMetaClass hasClassVarNamed: varName) ifTrue: [self error: 'class variable already defined!!']].
	newInstanceMethodDict keysDo: [:selector |
		(modifiedClass theNonMetaClass allSelectors includes: selector) ifTrue: [self error: 'instance method already defined!!']].
	newClassMethodDict keysDo: [:selector |
		(modifiedClass theMetaClass allSelectors includes: selector) ifTrue: [self error: 'class method already defined!!']]  
	! !

!PhClassModifier methodsFor: 'private' stamp: 'DanielGaldames 4/14/2011 01:29'!
compileClassMethod: aMethodString
	modifiedClass theMetaClass compile: aMethodString classified: '*phantom-generated-method'  ! !

!PhClassModifier methodsFor: 'private' stamp: 'DanielGaldames 4/14/2011 01:29'!
compileMethod: aMethodString
	modifiedClass compile: aMethodString classified: '*phantom-generated-method'  ! !

!PhClassModifier methodsFor: 'initialize-release' stamp: 'DanielGaldames 4/13/2011 20:19'!
initialize
	newInstanceMethodDict := Dictionary new.
	newClassMethodDict := Dictionary new.
	newInstanceVar := Set new.
	newClassVar := Set new.! !

!PhClassModifier methodsFor: 'install-uninstall' stamp: 'DanielGaldames 4/13/2011 20:52'!
install
	self checkExistence.
	newInstanceVar do: [:varName | modifiedClass theNonMetaClass addInstVarNamed: varName].
	newClassVar do: [:varName | modifiedClass theNonMetaClass addClassVarNamed: varName].
	newClassMethodDict do: [:methodString | self compileClassMethod: methodString ].
	newInstanceMethodDict do: [:methodString | self compileMethod: methodString ]! !

!PhClassModifier methodsFor: 'private' stamp: 'DanielGaldames 4/13/2011 16:17'!
modifiedClass
	^ modifiedClass! !

!PhClassModifier methodsFor: 'initialize-release' stamp: 'DanielGaldames 4/13/2011 16:33'!
on: aClass
	modifiedClass := aClass.! !

!PhClassModifier methodsFor: 'private' stamp: 'DanielGaldames 4/13/2011 20:23'!
removeClassVar: aClassVarName
	newClassVar remove: aClassVarName! !

!PhClassModifier methodsFor: 'private' stamp: 'DanielGaldames 4/13/2011 20:24'!
removeInstanceVar: anIntanceVarName
	newInstanceVar remove: anIntanceVarName! !

!PhClassModifier methodsFor: 'install-uninstall' stamp: 'DanielGaldames 4/13/2011 21:15'!
uninstall
	newClassMethodDict keysDo: [:selector | modifiedClass theMetaClass removeSelector: selector].
	newInstanceMethodDict keysDo: [:selector | modifiedClass theNonMetaClass removeSelector: selector].
	modifiedClass organization removeEmptyCategories.
	modifiedClass theMetaClass organization removeEmptyCategories.
	newClassVar do: [:classVarName | modifiedClass theNonMetaClass removeClassVarNamed: classVarName].
	newInstanceVar do: [:instanceVarName | modifiedClass theNonMetaClass removeInstVarNamed: instanceVarName]    ! !

Object subclass: #PhContext
	instanceVariableNames: 'receiver arguments selector method sender dynamicContext adviceDictionary adviceRunner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhContext commentStamp: 'DanielGaldames 4/14/2011 01:28' prior: 0!
This class is used to pass the context of execution to the advices on aspects!

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 4/18/2011 12:57'!
adviceDictionary
	^adviceDictionary ! !

!PhContext methodsFor: 'accessing' stamp: 'dgg 4/25/2011 09:07'!
adviceRunner
	^adviceRunner! !

!PhContext methodsFor: 'accessing' stamp: 'dgg 4/25/2011 09:06'!
adviceRunner: anAdviceRunner
	adviceRunner := anAdviceRunner! !

!PhContext methodsFor: 'advices' stamp: 'DanielGaldames 4/18/2011 13:57'!
afterAdvice
	^ adviceDictionary at: #after! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/26/2011 15:58'!
afterAdvice: adviceCollection
	adviceDictionary at: #after put: adviceCollection asArray.
	adviceRunner adviceGroup afterAdvice: (adviceDictionary at: #after)! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
arguments
	^ arguments! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
arguments: anObject
	arguments := anObject! !

!PhContext methodsFor: 'advices' stamp: 'DanielGaldames 4/18/2011 13:57'!
aroundAdvice
	^ adviceDictionary at: #around! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/26/2011 15:58'!
aroundAdvice: adviceCollection
	adviceDictionary at: #around put: adviceCollection asArray.
	adviceRunner adviceGroup aroundAdvice: (adviceDictionary at: #around)  ! !

!PhContext methodsFor: 'advices' stamp: 'DanielGaldames 4/18/2011 13:56'!
beforeAdvice
	^ adviceDictionary at: #before! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/26/2011 15:58'!
beforeAdvice: adviceCollection
	adviceDictionary at: #before put: adviceCollection asArray.
	adviceRunner adviceGroup beforeAdvice: (adviceDictionary at: #before)  ! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/25/2011 09:17'!
currentAdvice
	^ adviceDictionary at: (adviceRunner type)! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/25/2011 09:17'!
currentAdvice: anAdviceCollection continueAt: anIndex
	adviceDictionary at: (adviceRunner type) put: anAdviceCollection asArray.
	"wrapper perform: ((currentAdviceGroup, 'Advice:') asSymbol) with: (adviceDictionary at: currentAdviceGroup).
	wrapper currentAdviceIndex: (anIndex -1)"
	adviceRunner changeAdvice: anAdviceCollection continueAt: anIndex.! !

!PhContext methodsFor: 'advices' stamp: 'dgg 4/25/2011 09:21'!
currentAdviceIndex
	^adviceRunner index.! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/30/2011 19:40'!
dynamicContext
	^dynamicContext! !

!PhContext methodsFor: 'initialize-release' stamp: 'DanielGaldames 4/18/2011 12:57'!
initialize
	super initialize.
	dynamicContext := Dictionary new.
	adviceDictionary := Dictionary	newFrom: {
		#before -> nil.
		#after -> nil.
		#around -> nil}! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
method
	^ method! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
method: anObject
	method := anObject! !

!PhContext methodsFor: 'proceed' stamp: 'dgg 5/3/2011 21:55'!
proceed
	^ adviceRunner proceed.  ! !

!PhContext methodsFor: 'proceed' stamp: 'dgg 5/3/2011 21:55'!
proceed: args
	^ adviceRunner proceed: args.  ! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
receiver
	^ receiver! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
receiver: anObject
	receiver := anObject! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
selector
	^ selector! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/21/2011 02:02'!
selector: anObject
	selector := anObject! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 14:03'!
sender
	^ sender! !

!PhContext methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 14:03'!
sender: aSender
	sender := aSender! !

Object subclass: #PhMembrane
	instanceVariableNames: 'advisedMembranes pointcut registeredAspects advisingMembranes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 23:44'!
advise: aMembrane
	self advisedMembranes add: aMembrane.! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:42'!
advisedMembranes
	^ advisedMembranes ifNil: [advisedMembranes := Set new]! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:41'!
advisedMembranes: aMembraneSet
	advisedMembranes := aMembraneSet! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 23:42'!
advisingMembranes
	^ advisingMembranes ifNil: [advisingMembranes := Set new.]! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 23:44'!
install
	PhAspectWeaver installMembrane: self.
	self advisedMembranes do: [:each | each advisingMembranes add: self]! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 4/28/2011 17:14'!
isBaseMembrane
	^ self advisedMembranes isEmpty  ! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:40'!
pointcut
	^ pointcut! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:41'!
pointcut: aPhPointcut
	pointcut := aPhPointcut! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 21:26'!
registerAspect: anAspect
	self registeredAspects add: anAspect.
	anAspect onMembranes add: self.  
	PhAspectWeaver cleanCache.! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:42'!
registeredAspects
	^ registeredAspects ifNil: [registeredAspects := Set new]! !

!PhMembrane methodsFor: 'accessing' stamp: 'dgg 4/27/2011 19:42'!
registeredAspects: anAspectSet
	registeredAspects := anAspectSet! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 23:41'!
unAdvise: aMembrane
	self advisedMembranes remove: aMembrane.
! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 23:46'!
uninstall
	PhAspectWeaver uninstallMembrane: self.
	self registeredAspects do: [:each | each unregisterFrom: self].
	self advisedMembranes do: [:each | each advisingMembranes remove: self ifAbsent: nil]! !

!PhMembrane methodsFor: 'as yet unclassified' stamp: 'dgg 5/1/2011 22:47'!
unregisterAspect: anAspect
	self registeredAspects remove: anAspect ifAbsent: nil.
	anAspect onMembranes remove: self ifAbsent: nil.  
	PhAspectWeaver cleanCache.! !

Object subclass: #PhPointcut
	instanceVariableNames: 'selectors receivers pointcut composition selectIfMatch context precedence overrideRules dynamicContextBlock selectorsMessage packageRestriction'
	classVariableNames: 'Modifiers'
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhPointcut commentStamp: 'DanielGaldames 4/14/2011 01:58' prior: 0!
This class is used to define a pointcut.
The joinpoint model is based on messsage reception.
You have to define the receivers of the message, and the message (selectors).
Example:
	PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo:'.
	
	To select the metaclass:
	Example:
	PhPointcut new
		receivers: 'PhMockClass class';
		selectors: 'foo:'.
		
You can use some wildcards (like in AspectJ).
Example:
	Any receiver that have MockClass on his name.
	PhPointcut new
		receivers: '*MockClass*';
		selectors: 'foo:'.
		
	PhMockClass and all its subclasses:
	PhPointcut new
		receivers: 'PhMockClass+';
		selectors: 'foo:'.
		
On the selectors you can use '_' for any non argument selector, or '_:' any selector that receive an argument:
Example:
	Any selector of PhMockClass that receive a two argument message.
	PhPointcut new
		receivers: 'PhMockClass';
		selectors: '_:_:'.
		
You can pass groups of receivers and selectors as Arrays
Example:
	PhPointcut new
		receivers: #('Foo' 'Bar' 'SomeClass');
		selectors: #('foo:' 'bar').

You can also pass a PetitParser parser as receivers or selectors.
The classes definition that mathces the parser will be selected as receivers.
On the selectors, the selectors that matches the parser will be selected.

You can define the context to pass to the advices:
	PhPointcut new
		receivers: 'PhMockClass';
		selectors: 'foo:';
		context: #(#receiver #sender #selector #arguments #proceed).
		
You can override the precedence on defined on an aspect with overridePrecedence: #(...).
You can define a different precedence with precedence: #(...), this precedence has higher priority than the precedence defined on the aspects.
You can combine pointcuts using (and) pand: , (or) por: , and not.
Example: 
	(aPointcut1 pand: aPoincut2) por: aPointcut3.!

!PhPointcut classSide methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/14/2011 01:39'!
receviersParser
	^ PhReceiverParser new.! !

!PhPointcut classSide methodsFor: 'as yet unclassified' stamp: 'DanielGaldames 3/18/2011 22:29'!
selectorsParser
	^ PhSelectorParser new.! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/13/2011 16:15'!
composition
	^ composition! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/13/2011 16:15'!
composition: anObject
	composition := anObject! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 12:46'!
context
	^ context ifNil: [context := #()]  ! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 12:44'!
context: anArray 
	context := anArray! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/6/2011 02:23'!
createPointcutList
	| aSet classes |
	aSet := Set new.
	selectIfMatch
		ifTrue: [ self getClassList do: [ :each | aSet addAll: (self pointcutsThatMachtOf: each) ] ]
		ifFalse: [ self getClassList do: [ :each | aSet addAll: (self pointcutsThatDontMachtOf: each) ] ].
	^ aSet! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/30/2011 19:46'!
dynamicContext
	^dynamicContextBlock! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/30/2011 19:46'!
dynamicContext: aContextCreatorBlock
	dynamicContextBlock := aContextCreatorBlock! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/6/2011 02:20'!
getClassList
	packageRestriction isEmpty
		ifTrue: [ ^ SmalltalkImage current allClasses ]
		ifFalse: [ 
			| classes |
			classes := Set new.
			packageRestriction
				do: [ :packageName | 
					| package |
					package := PackageOrganizer default packageNamed: packageName ifAbsent: nil.
					package ifNotNil: [ classes addAll: (package classes select: [ :cl | (cl isKindOf: Trait) not ]) ] ].
			^ classes ]! !

!PhPointcut methodsFor: 'accessing' stamp: 'dgg 5/6/2011 02:24'!
getPointcuts
	| pointcuts |
	pointcuts := self createPointcutList.
	(self pointcut notNil) ifTrue: [
		pointcuts := pointcuts perform: (self composition) with: (self pointcut getPointcuts)  ].
	^pointcuts
	! !

!PhPointcut methodsFor: 'accessing' stamp: 'dgg 5/1/2011 16:38'!
includes: aSelector on: aClass
	(self
		packageRestrictionIncludes: aClass)
		ifTrue: [ 
			| classParser selectorParser |
			classParser := self parseReceivers: self receivers.
			selectorParser := self parseSelectors: self selectors.
			selectIfMatch
				ifTrue: [ 
					(classParser matches: aClass definition)
						ifTrue: [ ^ selectorParser matches: aSelector asString]
						ifFalse: [^ false] ]
				ifFalse: [
					(classParser matches: aClass definition)
						ifTrue: [ ^ (selectorParser matches: aSelector asString) not ]
						ifFalse: [^true]] ]
		ifFalse: [ ^ false ]! !

!PhPointcut methodsFor: 'initialize-release' stamp: 'dgg 5/6/2011 01:39'!
initialize
	pointcut := nil.
	composition := nil.
	selectIfMatch := true.
	overrideRules := false.
	dynamicContextBlock := nil.
	selectorsMessage := #allSelectors.
	packageRestriction := #()! !

!PhPointcut methodsFor: 'accessing' stamp: 'dgg 5/6/2011 01:42'!
localSelectors: aParserOrString
	self setLocal: true.  
	selectors := aParserOrString! !

!PhPointcut methodsFor: 'composing' stamp: 'dgg 5/6/2011 01:47'!
not
	^(PhPointcut new)
		selectors: self selectors;
		receivers: self receivers;
		pointcut: (self pointcut ifNotNil: [self pointcut not]);
		composition: ((self composition) ifNotNil: [
			(self composition = #intersecton:) ifTrue: [#union:] ifFalse: [#intersection:]]);
		selectIfMatch: self selectIfMatch not;
		context: self context;
		precedence: self precedence;
		override: self override;
		selectorsMessage: selectorsMessage;  
		restrictToPackages: packageRestriction;  
		yourself. ! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 23:00'!
override
	^overrideRules! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/31/2011 17:35'!
override: aBoolean
	overrideRules := aBoolean.! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 23:03'!
overridePrecedence: anArray 
	overrideRules := true.
	precedence := anArray! !

!PhPointcut methodsFor: 'accessing' stamp: 'dgg 5/1/2011 16:46'!
packageRestrictionIncludes: aClass
	packageRestriction isEmpty
		ifTrue: [ ^ true ]
		ifFalse: [ 
			packageRestriction
				do: [ :packageName | 
					| package |
					package := PackageOrganizer default packageNamed: packageName ifAbsent: nil.
					package
						ifNotNil: [ 
							(package includesClass: aClass)
								ifTrue: [ ^ true ] ] ].
			^ false ]! !

!PhPointcut methodsFor: 'composing' stamp: 'dgg 5/6/2011 01:47'!
pand: aPonitcut
	^(PhPointcut new)
		selectors: self selectors;
		receivers: self receivers;
		pointcut: aPonitcut;
		composition: #intersection:;
		selectIfMatch: self selectIfMatch;
		context: self context;
		precedence: self precedence;
		override: self override;
		selectorsMessage: selectorsMessage;
		restrictToPackages: packageRestriction;  
		yourself.    ! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 4/3/2011 18:44'!
parseReceivers:  receiversDefinition
	^ (receiversDefinition isPetitParser)
		ifTrue: [receivers] 
		ifFalse: [
			(receiversDefinition isString) 
			ifTrue: [PhPointcut receviersParser parse: receiversDefinition]
			ifFalse: [ 
				(receiversDefinition size = 1) 
					ifTrue: [PhPointcut receviersParser parse: (receiversDefinition at: 1)]
					ifFalse: [
						receiversDefinition reduce: [:a :b | 
							| parserA parserB |
							(a isPetitParser) ifTrue: [parserA := a] ifFalse: [parserA := PhPointcut receviersParser parse: a].
							(b isPetitParser) ifTrue: [parserB := b] ifFalse: [parserB := PhPointcut receviersParser parse: b]. 
							parserA / parserB]]   ]].   ! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/1/2011 20:53'!
parseSelectors:  selectorsDefinition
	^ (selectorsDefinition isPetitParser) 
		ifTrue: [selectorsDefinition] 
		ifFalse: [
			(selectorsDefinition isString)
				ifTrue: [PhPointcut selectorsParser parse: selectorsDefinition] 
				ifFalse: [
					(selectorsDefinition size = 1)
						ifTrue: [PhPointcut selectorsParser parse: (selectorsDefinition at: 1) asString] 
						ifFalse: [
							selectorsDefinition reduce: [:a :b |
								| pa pb |
								(a isPetitParser) ifTrue: [pa := a] ifFalse: [pa := PhPointcut selectorsParser parse: a].
								(b isPetitParser) ifTrue: [pb := b] ifFalse: [pb :=PhPointcut selectorsParser parse: b]. 
								pa / pb]]  ]].   
	"| receiversList sel |
	receiversList := self expandReceivers.
	sel := (selectorsString subStrings: ' ') collect: [:e | e asSymbol].
	^ receiversList collect: [:r | sel]"! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/13/2011 16:15'!
pointcut
	^ pointcut! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/13/2011 16:15'!
pointcut: anObject
	pointcut := anObject! !

!PhPointcut methodsFor: 'as yet unclassified' stamp: 'dgg 5/6/2011 02:16'!
pointcutsThatDontMachtOf: aClass
	| receiversParser selectorsParser aCollection |
	receiversParser := self parseReceivers: receivers.
	selectorsParser := self parseSelectors: selectors.
	aCollection := OrderedCollection new.
	(receiversParser matches: aClass definition) not
		ifTrue: [ (selectorsMessage value: aClass) do: [ :sel | aCollection add: (Array with: aClass with: sel) ] ]
		ifFalse: [ 
			(selectorsMessage value: aClass)
				do: [ :sel | 
					(selectorsParser matches: sel asString) not
						ifTrue: [ aCollection add: (Array with: aClass with: sel) ] ] ].
	(receiversParser matches: aClass class definition) not
		ifTrue: [ (selectorsMessage value: aClass class) do: [ :sel | aCollection add: (Array with: aClass class with: sel) ] ]
		ifFalse: [ 
			(selectorsMessage value: aClass class)
				do: [ :sel | 
					(selectorsParser matches: sel asString) not
						ifTrue: [ aCollection add: (Array with: aClass class with: sel) ] ] ].
	^ aCollection! !

!PhPointcut methodsFor: 'as yet unclassified' stamp: 'dgg 5/6/2011 02:13'!
pointcutsThatMachtOf: aClass
	| receiversParser selectorsParser aCollection |
	receiversParser := self parseReceivers: receivers.
	selectorsParser := self parseSelectors: selectors.
	aCollection := OrderedCollection new.
	(receiversParser matches: aClass definition)
		ifTrue: [ 
			(selectorsMessage value: aClass)
				do: [ :sel | 
					(selectorsParser matches: sel asString)
						ifTrue: [ aCollection add: (Array with: aClass with: sel) ] ] ].
	(receiversParser matches: aClass class definition)
		ifTrue: [ 
			(selectorsMessage value: aClass class)
				do: [ :sel | 
					(selectorsParser matches: sel asString)
						ifTrue: [ aCollection add: (Array with: aClass class with: sel) ] ] ].
	^ aCollection! !

!PhPointcut methodsFor: 'composing' stamp: 'dgg 5/6/2011 01:47'!
por: aPointcut
	^(PhPointcut new)
		selectors: self selectors;
		receivers: self receivers;
		pointcut: aPointcut;
		composition: #union:;
		selectIfMatch: self selectIfMatch;
		context: self context;
		precedence: self precedence;
		override: self override;
		selectorsMessage: selectorsMessage;
		restrictToPackages: packageRestriction;  
		yourself.  ! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/28/2011 22:45'!
precedence
	^precedence ifNil: [precedence := #()]! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 4/3/2011 00:38'!
precedence: anArray
	overrideRules := false.
	precedence := anArray! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/18/2011 19:38'!
receivers
	^receivers ifNil: [#any asParser star not]! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/12/2011 20:10'!
receivers: aParserOrString
	receivers := aParserOrString! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 4/20/2011 23:35'!
restrictToPackages: aPackageNameArray
	packageRestriction := aPackageNameArray! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/18/2011 22:37'!
selectIfMatch
	^selectIfMatch! !

!PhPointcut methodsFor: 'private' stamp: 'DanielGaldames 3/18/2011 22:37'!
selectIfMatch: aBoolean
	selectIfMatch := aBoolean! !

!PhPointcut methodsFor: 'accessing' stamp: 'DanielGaldames 3/18/2011 19:38'!
selectors
	^selectors ifNil: [#any asParser star not]! !

!PhPointcut methodsFor: 'accessing' stamp: 'dgg 5/6/2011 01:42'!
selectors: aParserOrString
	self setLocal: false.  
	selectors := aParserOrString! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/6/2011 01:46'!
selectorsMessage
	^ selectorsMessage! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/6/2011 01:46'!
selectorsMessage: aSelectorsMessage
	selectorsMessage :=  aSelectorsMessage! !

!PhPointcut methodsFor: 'private' stamp: 'dgg 5/6/2011 01:42'!
setLocal: aBoolean
	aBoolean 
		ifTrue: [selectorsMessage := #localSelectors]  
		ifFalse: [selectorsMessage := #allSelectors] ! !

Object subclass: #PhPrecedenceRulesProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Core'!

!PhPrecedenceRulesProcessor commentStamp: 'DanielGaldames 4/14/2011 01:55' prior: 0!
This class is reponsable to process the precedence rules and order the advices action on a pointcut.!

!PhPrecedenceRulesProcessor methodsFor: 'graph generation' stamp: 'DanielGaldames 3/25/2011 13:56'!
getGraphFrom: aNodeCollection
	| graph |
	graph := AspectGraph new.
	aNodeCollection reduce: [:parents :children |
		graph addAll: parents.
		graph addAll: children.
		parents do: [:node |
			children do: [:child |
				(graph nodeSet like: node) addNode: (graph nodeSet like: child)]].
		children.].
	^graph.! !

!PhPrecedenceRulesProcessor methodsFor: 'aspect sorting' stamp: 'DanielGaldames 3/29/2011 22:28'!
getSortedAspectsWith: aRulesSet
	| graphs graph |
	(aRulesSet isEmpty) ifTrue: [^#()].
	graphs := OrderedCollection new.
	aRulesSet collect: [:rulesArray |
		| nodeCollection |
		nodeCollection := OrderedCollection new.
		rulesArray do: [:rule | nodeCollection add: (PhPrecedenceRulesParser new parse: rule)].
		"lipieza por relga *"
		rulesArray withIndexDo: [:elem :index |
			(elem = '*') ifTrue: [
				nodeCollection withIndexDo: [:col :idx |
					((rulesArray at: idx) = '*') not ifTrue: [
						nodeCollection at: index put: ((nodeCollection at: index) difference: col)]]]].
		graphs add: nodeCollection].
	graph := OrderedCollection new.
	graphs do: [:each |
		| g |
		g := self getGraphFrom: each.
		graph add: g].
	graph := graph reduce: [:a :b | a merge: b].
	(graph existsCycle) ifTrue: [self error: 'Conflict detected in rules set'].
	graph setPrecedence.
	^ (graph nodeSet asArray sort: [:a :b | a precedence < b precedence]) collect: [:each | each aspect].! !
PhAspectWeaver initialize!
